@@ -39,21 +39,16 @@ WebInspector.ProfileType.prototype = {
         return "";
     },
 
-    get buttonStyle()
-    {
-        return undefined;
-    },
-
-    get buttonCaption()
-    {
-        return this.name;
-    },
-
     get id()
     {
         return this._id;
     },
 
+    get treeItemTitle()
+    {
+        return this._name;
+    },
+
     get name()
     {
         return this._name;
@@ -70,7 +65,11 @@ WebInspector.ProfileType.prototype = {
         return profile._profileView;
     },
 
-    get welcomeMessage()
+    reset: function()
+    {
+    },
+
+    get description()
     {
         return "";
     },
@@ -88,29 +87,45 @@ WebInspector.ProfileType.prototype = {
     }
 }
 
+WebInspector.registerLinkifierPlugin(function(title)
+{
+    var profileStringMatches = WebInspector.ProfileType.URLRegExp.exec(title);
+    if (profileStringMatches)
+        title = WebInspector.panels.profiles.displayTitleForProfileLink(profileStringMatches[2], profileStringMatches[1]);
+    return title;
+});
+
 WebInspector.ProfilesPanel = function()
 {
     WebInspector.Panel.call(this, "profiles");
+    this.registerRequiredCSS("panelEnablerView.css");
+    this.registerRequiredCSS("heapProfiler.css");
+    this.registerRequiredCSS("profilesPanel.css");
 
-    this.createSidebar();
+    this.createSplitViewWithSidebarTree();
+
+    this.profilesItemTreeElement = new WebInspector.ProfilesSidebarTreeElement(this);
+    this.sidebarTree.appendChild(this.profilesItemTreeElement);
 
     this._profileTypesByIdMap = {};
-    this._profileTypeButtonsByIdMap = {};
 
     var panelEnablerHeading = WebInspector.UIString("You need to enable profiling before you can use the Profiles panel.");
     var panelEnablerDisclaimer = WebInspector.UIString("Enabling profiling will make scripts run slower.");
     var panelEnablerButton = WebInspector.UIString("Enable Profiling");
     this.panelEnablerView = new WebInspector.PanelEnablerView("profiles", panelEnablerHeading, panelEnablerDisclaimer, panelEnablerButton);
-    this.panelEnablerView.addEventListener("enable clicked", this._enableProfiling, this);
-
-    this.element.appendChild(this.panelEnablerView.element);
+    this.panelEnablerView.addEventListener("enable clicked", this.enableProfiler, this);
 
     this.profileViews = document.createElement("div");
     this.profileViews.id = "profile-views";
-    this.element.appendChild(this.profileViews);
+    this.splitView.mainElement.appendChild(this.profileViews);
 
     this.enableToggleButton = new WebInspector.StatusBarButton("", "enable-toggle-status-bar-item");
     this.enableToggleButton.addEventListener("click", this._toggleProfiling.bind(this), false);
+    if (!Capabilities.profilerCausesRecompilation)
+        this.enableToggleButton.element.addStyleClass("hidden");
+
+    this.recordButton = new WebInspector.StatusBarButton("", "record-profile-status-bar-item");
+    this.recordButton.addEventListener("click", this.toggleRecordButton, this);
 
     this.clearResultsButton = new WebInspector.StatusBarButton(WebInspector.UIString("Clear all profiles."), "clear-status-bar-item");
     this.clearResultsButton.addEventListener("click", this._clearProfiles.bind(this), false);
@@ -118,32 +133,28 @@ WebInspector.ProfilesPanel = function()
     this.profileViewStatusBarItemsContainer = document.createElement("div");
     this.profileViewStatusBarItemsContainer.className = "status-bar-items";
 
-    this.welcomeView = new WebInspector.WelcomeView("profiles", WebInspector.UIString("Welcome to the Profiles panel"));
-    this.element.appendChild(this.welcomeView.element);
-
     this._profiles = [];
-    this._profilerEnabled = Preferences.profilerAlwaysEnabled;
+    this._profilerEnabled = !Capabilities.profilerCausesRecompilation;
+
+    this._launcherView = new WebInspector.ProfileLauncherView(this);
+    this._launcherView.addEventListener(WebInspector.ProfileLauncherView.EventTypes.ProfileTypeSelected, this._onProfileTypeSelected, this);
     this._reset();
+    this._launcherView.setUpEventListeners();
 
     this._registerProfileType(new WebInspector.CPUProfileType());
-    if (Preferences.heapProfilerPresent) {
-        if (!Preferences.detailedHeapProfiles)
-            this._registerProfileType(new WebInspector.HeapSnapshotProfileType());
-        else
-            this._registerProfileType(new WebInspector.DetailedHeapshotProfileType());
-    }
+    this._registerProfileType(new WebInspector.CSSSelectorProfileType());
+    if (Capabilities.heapProfilerPresent)
+        this._registerProfileType(new WebInspector.DetailedHeapshotProfileType());
 
-    InspectorBackend.registerDomainDispatcher("Profiler", new WebInspector.ProfilerDispatcher(this));
+    InspectorBackend.registerProfilerDispatcher(new WebInspector.ProfilerDispatcher(this));
 
-    if (Preferences.profilerAlwaysEnabled || WebInspector.settings.profilerEnabled)
-        ProfilerAgent.enable();
-    else {
-        function onProfilerEnebled(error, value) {
-            if (value)
-                this._profilerWasEnabled();
-        }
-        ProfilerAgent.isEnabled(onProfilerEnebled.bind(this));
-    }
+    if (!Capabilities.profilerCausesRecompilation || WebInspector.settings.profilerEnabled.get())
+        ProfilerAgent.enable(this._profilerWasEnabled.bind(this));
+}
+
+WebInspector.ProfilesPanel.EventTypes = {
+    ProfileStarted: "profile-started",
+    ProfileFinished: "profile-finished"
 }
 
 WebInspector.ProfilesPanel.prototype = {
@@ -154,30 +165,17 @@ WebInspector.ProfilesPanel.prototype = {
 
     get statusBarItems()
     {
-        function clickHandler(profileType, buttonElement)
-        {
-            profileType.buttonClicked.call(profileType);
-            this.updateProfileTypeButtons();
-        }
-
-        var items = [this.enableToggleButton.element];
-        // FIXME: Generate a single "combo-button".
-        for (var typeId in this._profileTypesByIdMap) {
-            var profileType = this.getProfileType(typeId);
-            if (profileType.buttonStyle) {
-                var button = new WebInspector.StatusBarButton(profileType.buttonTooltip, profileType.buttonStyle, profileType.buttonCaption);
-                this._profileTypeButtonsByIdMap[typeId] = button.element;
-                button.element.addEventListener("click", clickHandler.bind(this, profileType, button.element), false);
-                items.push(button.element);
-            }
-        }
-        items.push(this.clearResultsButton.element, this.profileViewStatusBarItemsContainer);
-        return items;
+        return [this.enableToggleButton.element, this.recordButton.element, this.clearResultsButton.element, this.profileViewStatusBarItemsContainer];
     },
 
-    show: function()
+    toggleRecordButton: function()
     {
-        WebInspector.Panel.prototype.show.call(this);
+        this._selectedProfileType.buttonClicked();
+    },
+
+    wasShown: function()
+    {
+        WebInspector.Panel.prototype.wasShown.call(this);
         this._populateProfiles();
     },
 
@@ -189,7 +187,7 @@ WebInspector.ProfilesPanel.prototype = {
         this._profilerEnabled = true;
 
         this._reset();
-        if (this.visible)
+        if (this.isShowing())
             this._populateProfiles();
     },
 
@@ -202,26 +200,39 @@ WebInspector.ProfilesPanel.prototype = {
         this._reset();
     },
 
+    _onProfileTypeSelected: function(event)
+    {
+        this._selectedProfileType = event.data;
+        this.recordButton.title = this._selectedProfileType.buttonTooltip;
+    },
+
     _reset: function()
     {
         WebInspector.Panel.prototype.reset.call(this);
 
         for (var i = 0; i < this._profiles.length; ++i) {
             var view = this._profiles[i]._profileView;
-            if (view && ("dispose" in view))
-                view.dispose();
-            delete this._profiles[i]._profileView;
-            var profile = this._profiles[i];
-            if (profile.nodes) {
-                delete profile.nodes;
-                delete profile.strings;
+            if (view) {
+                view.detach();
+                if ("dispose" in view)
+                    view.dispose();
             }
+
+            delete this._profiles[i]._profileView;
         }
         delete this.visibleView;
 
         delete this.currentQuery;
         this.searchCanceled();
 
+        for (var id in this._profileTypesByIdMap) {
+            var profileType = this._profileTypesByIdMap[id];
+            var treeElement = profileType.treeElement;
+            treeElement.removeChildren();
+            treeElement.hidden = true;
+            profileType.reset();
+        }
+
         this._profiles = [];
         this._profilesIdMap = {};
         this._profileGroups = {};
@@ -230,17 +241,23 @@ WebInspector.ProfilesPanel.prototype = {
 
         this.sidebarTreeElement.removeStyleClass("some-expandable");
 
-        for (var typeId in this._profileTypesByIdMap)
-            this.getProfileType(typeId).treeElement.removeChildren();
-
         this.profileViews.removeChildren();
-
         this.profileViewStatusBarItemsContainer.removeChildren();
 
         this.removeAllListeners();
+        this._launcherView.setUpEventListeners();
 
         this._updateInterface();
-        this.welcomeView.show();
+        this.profilesItemTreeElement.select();
+        this._showLauncherView();
+    },
+
+    _showLauncherView: function()
+    {
+        this.closeVisibleView();
+        this.profileViewStatusBarItemsContainer.removeChildren();
+        this._launcherView.show(this.splitView.mainElement);
+        this.visibleView = this._launcherView;
     },
 
     _clearProfiles: function()
@@ -252,34 +269,10 @@ WebInspector.ProfilesPanel.prototype = {
     _registerProfileType: function(profileType)
     {
         this._profileTypesByIdMap[profileType.id] = profileType;
-        profileType.treeElement = new WebInspector.SidebarSectionTreeElement(profileType.name, null, true);
+        this._launcherView.addProfileType(profileType);
+        profileType.treeElement = new WebInspector.SidebarSectionTreeElement(profileType.treeItemTitle, null, true);
+        profileType.treeElement.hidden = true;
         this.sidebarTree.appendChild(profileType.treeElement);
-        profileType.treeElement.expand();
-        this._addWelcomeMessage(profileType);
-    },
-
-    _addWelcomeMessage: function(profileType)
-    {
-        var message = profileType.welcomeMessage;
-        // Message text is supposed to have a '%s' substring as a placeholder
-        // for a status bar button. If it is there, we split the message in two
-        // parts, and insert the button between them.
-        var buttonPos = message.indexOf("%s");
-        if (buttonPos > -1) {
-            var container = document.createDocumentFragment();
-            var part1 = document.createElement("span");
-            part1.innerHTML = message.substr(0, buttonPos);
-            container.appendChild(part1);
-     
-            var button = new WebInspector.StatusBarButton(profileType.buttonTooltip, profileType.buttonStyle, profileType.buttonCaption);
-            container.appendChild(button.element);
-       
-            var part2 = document.createElement("span");
-            part2.innerHTML = message.substr(buttonPos + 2);
-            container.appendChild(part2);
-            this.welcomeView.addMessage(container);
-        } else
-            this.welcomeView.addMessage(message);
     },
 
     _makeKey: function(text, profileTypeId)
@@ -287,18 +280,15 @@ WebInspector.ProfilesPanel.prototype = {
         return escape(text) + '/' + escape(profileTypeId);
     },
 
-    _addProfileHeader: function(profile)
+    addProfileHeader: function(profile)
     {
-        if (this.hasTemporaryProfile(profile.typeId)) {
-            if (profile.typeId === WebInspector.CPUProfileType.TypeId)
-                this._removeProfileHeader(this._temporaryRecordingProfile);
-            else
-                this._removeProfileHeader(this._temporaryTakingSnapshot);
-        }
+        if (this.hasTemporaryProfile(profile.typeId))
+            this._removeTemporaryProfile();
 
         var typeId = profile.typeId;
         var profileType = this.getProfileType(typeId);
         var sidebarParent = profileType.treeElement;
+        sidebarParent.hidden = false;
         var small = false;
         var alternateTitle;
 
@@ -326,10 +316,8 @@ WebInspector.ProfilesPanel.prototype = {
                 var selected = group[0]._profilesTreeElement.selected;
                 sidebarParent.removeChild(group[0]._profilesTreeElement);
                 group._profilesTreeElement.appendChild(group[0]._profilesTreeElement);
-                if (selected) {
-                    group[0]._profilesTreeElement.select();
-                    group[0]._profilesTreeElement.reveal();
-                }
+                if (selected)
+                    group[0]._profilesTreeElement.revealAndSelect();
 
                 group[0]._profilesTreeElement.small = true;
                 group[0]._profilesTreeElement.mainTitle = WebInspector.UIString("Run %d", 1);
@@ -345,7 +333,7 @@ WebInspector.ProfilesPanel.prototype = {
         }
 
         var profileTreeElement = profileType.createSidebarTreeElementForProfile(profile);
-        profile.sideBarElement = profileTreeElement;
+        profile.sidebarElement = profileTreeElement;
         profileTreeElement.small = small;
         if (alternateTitle)
             profileTreeElement.mainTitle = alternateTitle;
@@ -353,11 +341,24 @@ WebInspector.ProfilesPanel.prototype = {
 
         sidebarParent.appendChild(profileTreeElement);
         if (!profile.isTemporary) {
-            this.welcomeView.hide();
             if (!this.visibleView)
                 this.showProfile(profile);
             this.dispatchEventToListeners("profile added");
+            delete this._temporaryRecordingProfile;
+            this.dispatchEventToListeners(WebInspector.ProfilesPanel.EventTypes.ProfileFinished);
+            this.recordButton.toggled = false;
+        } else {
+            this.dispatchEventToListeners(WebInspector.ProfilesPanel.EventTypes.ProfileStarted);
+            this.recordButton.toggled = true;
         }
+
+        this.recordButton.title = this._selectedProfileType.buttonTooltip;
+    },
+
+    _removeTemporaryProfile: function()
+    {
+        this._removeProfileHeader(this._temporaryRecordingProfile);
+        delete this._temporaryRecordingProfile;
     },
 
     _removeProfileHeader: function(profile)
@@ -400,8 +401,9 @@ WebInspector.ProfilesPanel.prototype = {
 
         view.show(this.profileViews);
 
-        profile._profilesTreeElement.select(true);
-        profile._profilesTreeElement.reveal();
+        profile._profilesTreeElement._suppressOnSelect = true;
+        profile._profilesTreeElement.revealAndSelect();
+        delete profile._profilesTreeElement._suppressOnSelect;
 
         this.visibleView = view;
 
@@ -446,63 +448,49 @@ WebInspector.ProfilesPanel.prototype = {
 
     loadHeapSnapshot: function(uid, callback)
     {
-        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.DetailedHeapshotProfileType.TypeId)];
         if (!profile)
             return;
 
-        if (profile._loaded)
-            callback(profile);
-        else if (profile._is_loading)
-            profile._callbacks.push(callback);
-        else {
-            profile._is_loading = true;
-            profile._callbacks = [callback];
-            profile._json = "";
-            profile.sideBarElement.subtitle = WebInspector.UIString("Loading…");
+        if (!profile.proxy) {
+            function setProfileWait(event) {
+                profile.sidebarElement.wait = event.data;
+            }
+            var worker = new WebInspector.HeapSnapshotWorker();
+            worker.addEventListener("wait", setProfileWait, this);
+            profile.proxy = worker.createObject("WebInspector.HeapSnapshotLoader");
+        }
+        var proxy = profile.proxy;
+        if (proxy.startLoading(callback)) {
+            profile.sidebarElement.subtitle = WebInspector.UIString("Loading\u2026");
+            profile.sidebarElement.wait = true;
             ProfilerAgent.getProfile(profile.typeId, profile.uid);
         }
     },
 
     _addHeapSnapshotChunk: function(uid, chunk)
     {
-        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
-        if (!profile || profile._loaded || !profile._is_loading)
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.DetailedHeapshotProfileType.TypeId)];
+        if (!profile || !profile.proxy)
             return;
-
-        profile._json += chunk;
+        profile.proxy.pushJSONChunk(chunk);
     },
 
     _finishHeapSnapshot: function(uid)
     {
-        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.HeapSnapshotProfileType.TypeId)];
-        if (!profile || profile._loaded || !profile._is_loading)
+        var profile = this._profilesIdMap[this._makeKey(uid, WebInspector.DetailedHeapshotProfileType.TypeId)];
+        if (!profile || !profile.proxy)
             return;
-
-        var callbacks = profile._callbacks;
-        delete profile._callbacks;
-        profile.sideBarElement.subtitle = WebInspector.UIString("Parsing…");
-        window.setTimeout(doParse, 0);
-
-        function doParse()
+        var proxy = profile.proxy;
+        function parsed(snapshotProxy)
         {
-            var loadedSnapshot = JSON.parse(profile._json);
-            delete profile._json;
-            delete profile._is_loading;
-            profile._loaded = true;
-            profile.sideBarElement.subtitle = "";
-
-            if (!Preferences.detailedHeapProfiles && WebInspector.DetailedHeapshotView.prototype.isDetailedSnapshot(loadedSnapshot)) {
-                WebInspector.panels.profiles._enableDetailedHeapProfiles(false);
-                return;
-            }
-
-            if (!Preferences.detailedHeapProfiles)
-                WebInspector.HeapSnapshotView.prototype.processLoadedSnapshot(profile, loadedSnapshot);
-            else
-                WebInspector.DetailedHeapshotView.prototype.processLoadedSnapshot(profile, loadedSnapshot);
-            for (var i = 0; i < callbacks.length; ++i)
-                callbacks[i](profile);
+            profile.proxy = snapshotProxy;
+            profile.sidebarElement.subtitle = Number.bytesToString(snapshotProxy.totalSize);
+            profile.sidebarElement.wait = false;
+            snapshotProxy.worker.startCheckingForLongRunningCalls();
         }
+        if (proxy.finishLoading(parsed))
+            profile.sidebarElement.subtitle = WebInspector.UIString("Parsing\u2026");
     },
 
     showView: function(view)
@@ -523,21 +511,10 @@ WebInspector.ProfilesPanel.prototype = {
         this.showProfile(this._profilesIdMap[this._makeKey(match[3], match[1])]);
     },
 
-    updateProfileTypeButtons: function()
-    {
-        for (var typeId in this._profileTypeButtonsByIdMap) {
-            var buttonElement = this._profileTypeButtonsByIdMap[typeId];
-            var profileType = this.getProfileType(typeId);
-            buttonElement.className = profileType.buttonStyle;
-            buttonElement.title = profileType.buttonTooltip;
-            // FIXME: Apply profileType.buttonCaption once captions are added to button controls.
-        }
-    },
-
     closeVisibleView: function()
     {
         if (this.visibleView)
-            this.visibleView.hide();
+            this.visibleView.detach();
         delete this.visibleView;
     },
 
@@ -558,11 +535,156 @@ WebInspector.ProfilesPanel.prototype = {
                 // incremented twice as often as it's displayed
                 title += " " + WebInspector.UIString("Run %d", (groupNumber + 1) / 2);
         }
-        
+
         return title;
     },
 
-    get searchableViews()
+    performSearch: function(query)
+    {
+        this.searchCanceled();
+
+        var searchableViews = this._searchableViews();
+        if (!searchableViews || !searchableViews.length)
+            return;
+
+        var parentElement = this.viewsContainerElement;
+        var visibleView = this.visibleView;
+        var sortFuction = this.searchResultsSortFunction;
+
+        var matchesCountUpdateTimeout = null;
+
+        function updateMatchesCount()
+        {
+            WebInspector.searchController.updateSearchMatchesCount(this._totalSearchMatches, this);
+            matchesCountUpdateTimeout = null;
+        }
+
+        function updateMatchesCountSoon()
+        {
+            if (matchesCountUpdateTimeout)
+                return;
+            // Update the matches count every half-second so it doesn't feel twitchy.
+            matchesCountUpdateTimeout = setTimeout(updateMatchesCount.bind(this), 500);
+        }
+
+        function finishedCallback(view, searchMatches)
+        {
+            if (!searchMatches)
+                return;
+
+            this._totalSearchMatches += searchMatches;
+            this._searchResults.push(view);
+
+            if (sortFuction)
+                this._searchResults.sort(sortFuction);
+
+            if (this.searchMatchFound)
+                this.searchMatchFound(view, searchMatches);
+
+            updateMatchesCountSoon.call(this);
+
+            if (view === visibleView)
+                view.jumpToFirstSearchResult();
+        }
+
+        var i = 0;
+        var panel = this;
+        var boundFinishedCallback = finishedCallback.bind(this);
+        var chunkIntervalIdentifier = null;
+
+        // Split up the work into chunks so we don't block the
+        // UI thread while processing.
+
+        function processChunk()
+        {
+            var view = searchableViews[i];
+
+            if (++i >= searchableViews.length) {
+                if (panel._currentSearchChunkIntervalIdentifier === chunkIntervalIdentifier)
+                    delete panel._currentSearchChunkIntervalIdentifier;
+                clearInterval(chunkIntervalIdentifier);
+            }
+
+            if (!view)
+                return;
+
+            view.currentQuery = query;
+            view.performSearch(query, boundFinishedCallback);
+        }
+
+        processChunk();
+
+        chunkIntervalIdentifier = setInterval(processChunk, 25);
+        this._currentSearchChunkIntervalIdentifier = chunkIntervalIdentifier;
+    },
+
+    jumpToNextSearchResult: function()
+    {
+        if (!this.showView || !this._searchResults || !this._searchResults.length)
+            return;
+
+        var showFirstResult = false;
+
+        this._currentSearchResultIndex = this._searchResults.indexOf(this.visibleView);
+        if (this._currentSearchResultIndex === -1) {
+            this._currentSearchResultIndex = 0;
+            showFirstResult = true;
+        }
+
+        var currentView = this._searchResults[this._currentSearchResultIndex];
+
+        if (currentView.showingLastSearchResult()) {
+            if (++this._currentSearchResultIndex >= this._searchResults.length)
+                this._currentSearchResultIndex = 0;
+            currentView = this._searchResults[this._currentSearchResultIndex];
+            showFirstResult = true;
+        }
+
+        if (currentView !== this.visibleView) {
+            this.showView(currentView);
+            WebInspector.searchController.focusSearchField();
+        }
+
+        if (showFirstResult)
+            currentView.jumpToFirstSearchResult();
+        else
+            currentView.jumpToNextSearchResult();
+    },
+
+    jumpToPreviousSearchResult: function()
+    {
+        if (!this.showView || !this._searchResults || !this._searchResults.length)
+            return;
+
+        var showLastResult = false;
+
+        this._currentSearchResultIndex = this._searchResults.indexOf(this.visibleView);
+        if (this._currentSearchResultIndex === -1) {
+            this._currentSearchResultIndex = 0;
+            showLastResult = true;
+        }
+
+        var currentView = this._searchResults[this._currentSearchResultIndex];
+
+        if (currentView.showingFirstSearchResult()) {
+            if (--this._currentSearchResultIndex < 0)
+                this._currentSearchResultIndex = (this._searchResults.length - 1);
+            currentView = this._searchResults[this._currentSearchResultIndex];
+            showLastResult = true;
+        }
+
+        if (currentView !== this.visibleView) {
+            this.showView(currentView);
+            WebInspector.searchController.focusSearchField();
+        }
+
+        if (showLastResult)
+            currentView.jumpToLastSearchResult();
+        else
+            currentView.jumpToPreviousSearchResult();
+    },
+
+    _searchableViews: function()
     {
         var views = [];
 
@@ -587,9 +709,27 @@ WebInspector.ProfilesPanel.prototype = {
         view.profile._profilesTreeElement.searchMatches = matches;
     },
 
-    searchCanceled: function(startingNewSearch)
+    searchCanceled: function()
     {
-        WebInspector.Panel.prototype.searchCanceled.call(this, startingNewSearch);
+        if (this._searchResults) {
+            for (var i = 0; i < this._searchResults.length; ++i) {
+                var view = this._searchResults[i];
+                if (view.searchCanceled)
+                    view.searchCanceled();
+                delete view.currentQuery;
+            }
+        }
+
+        WebInspector.Panel.prototype.searchCanceled.call(this);
+
+        if (this._currentSearchChunkIntervalIdentifier) {
+            clearInterval(this._currentSearchChunkIntervalIdentifier);
+            delete this._currentSearchChunkIntervalIdentifier;
+        }
+
+        this._totalSearchMatches = 0;
+        this._currentSearchResultIndex = 0;
+        this._searchResults = [];
 
         if (!this._profiles)
             return;
@@ -606,37 +746,47 @@ WebInspector.ProfilesPanel.prototype = {
         if (this._profilerEnabled) {
             this.enableToggleButton.title = WebInspector.UIString("Profiling enabled. Click to disable.");
             this.enableToggleButton.toggled = true;
-            for (var typeId in this._profileTypeButtonsByIdMap)
-                this._profileTypeButtonsByIdMap[typeId].removeStyleClass("hidden");
+            this.recordButton.visible = true;
             this.profileViewStatusBarItemsContainer.removeStyleClass("hidden");
             this.clearResultsButton.element.removeStyleClass("hidden");
-            this.panelEnablerView.visible = false;
+            this.panelEnablerView.detach();
         } else {
             this.enableToggleButton.title = WebInspector.UIString("Profiling disabled. Click to enable.");
             this.enableToggleButton.toggled = false;
-            for (var typeId in this._profileTypeButtonsByIdMap)
-                this._profileTypeButtonsByIdMap[typeId].addStyleClass("hidden");
+            this.recordButton.visible = false;
             this.profileViewStatusBarItemsContainer.addStyleClass("hidden");
             this.clearResultsButton.element.addStyleClass("hidden");
-            this.panelEnablerView.visible = true;
+            this.panelEnablerView.show(this.element);
         }
     },
 
-    _enableProfiling: function()
+    get profilerEnabled()
+    {
+        return this._profilerEnabled;
+    },
+
+    enableProfiler: function()
     {
         if (this._profilerEnabled)
             return;
         this._toggleProfiling(this.panelEnablerView.alwaysEnabled);
     },
 
+    disableProfiler: function()
+    {
+        if (!this._profilerEnabled)
+            return;
+        this._toggleProfiling(this.panelEnablerView.alwaysEnabled);
+    },
+
     _toggleProfiling: function(optionalAlways)
     {
         if (this._profilerEnabled) {
-            WebInspector.settings.profilerEnabled = false;
-            ProfilerAgent.disable();
+            WebInspector.settings.profilerEnabled.set(false);
+            ProfilerAgent.disable(this._profilerWasDisabled.bind(this));
         } else {
-            WebInspector.settings.profilerEnabled = !!optionalAlways;
-            ProfilerAgent.enable();
+            WebInspector.settings.profilerEnabled.set(!!optionalAlways);
+            ProfilerAgent.enable(this._profilerWasEnabled.bind(this));
         }
     },
 
@@ -652,7 +802,7 @@ WebInspector.ProfilesPanel.prototype = {
             var profileHeadersLength = profileHeaders.length;
             for (var i = 0; i < profileHeadersLength; ++i)
                 if (!this.hasProfile(profileHeaders[i]))
-                   this._addProfileHeader(profileHeaders[i]);
+                   this.addProfileHeader(profileHeaders[i]);
         }
 
         ProfilerAgent.getProfileHeaders(populateCallback.bind(this));
@@ -660,114 +810,59 @@ WebInspector.ProfilesPanel.prototype = {
         this._profilesWereRequested = true;
     },
 
-    updateMainViewWidth: function(width)
+    sidebarResized: function(event)
     {
-        this.welcomeView.element.style.left = width + "px";
-        this.profileViews.style.left = width + "px";
-        this.profileViewStatusBarItemsContainer.style.left = Math.max(155, width) + "px";
-        this.resize();
+        var width = event.data;
+        // Min width = <number of buttons on the left> * 31
+        this.profileViewStatusBarItemsContainer.style.left = Math.max(5 * 31, width) + "px";
     },
 
-    _setRecordingProfile: function(isProfiling)
+    setRecordingProfile: function(profileType, isProfiling)
     {
-        this.getProfileType(WebInspector.CPUProfileType.TypeId).setRecordingProfile(isProfiling);
-        if (this.hasTemporaryProfile(WebInspector.CPUProfileType.TypeId) !== isProfiling) {
+        this.getProfileType(profileType).setRecordingProfile(isProfiling);
+        if (this.hasTemporaryProfile(profileType) !== isProfiling) {
             if (!this._temporaryRecordingProfile) {
                 this._temporaryRecordingProfile = {
-                    typeId: WebInspector.CPUProfileType.TypeId,
-                    title: WebInspector.UIString("Recording…"),
+                    typeId: profileType,
+                    title: WebInspector.UIString("Recording\u2026"),
                     uid: -1,
                     isTemporary: true
                 };
             }
-            if (isProfiling)
-                this._addProfileHeader(this._temporaryRecordingProfile);
-            else
-                this._removeProfileHeader(this._temporaryRecordingProfile);
+            if (isProfiling) {
+                this.addProfileHeader(this._temporaryRecordingProfile);
+                if (profileType === WebInspector.CPUProfileType.TypeId)
+                    WebInspector.userMetrics.ProfilesCPUProfileTaken.record();
+            } else
+                this._removeTemporaryProfile()
         }
-        this.updateProfileTypeButtons();
     },
 
-    takeHeapSnapshot: function(detailed)
+    takeHeapSnapshot: function()
     {
-        if (!this.hasTemporaryProfile(WebInspector.HeapSnapshotProfileType.TypeId)) {
-            if (!this._temporaryTakingSnapshot) {
-                this._temporaryTakingSnapshot = {
-                    typeId: WebInspector.HeapSnapshotProfileType.TypeId,
-                    title: WebInspector.UIString("Snapshotting…"),
+        if (!this.hasTemporaryProfile(WebInspector.DetailedHeapshotProfileType.TypeId)) {
+            if (!this._temporaryRecordingProfile) {
+                this._temporaryRecordingProfile = {
+                    typeId: WebInspector.DetailedHeapshotProfileType.TypeId,
+                    title: WebInspector.UIString("Snapshotting\u2026"),
                     uid: -1,
                     isTemporary: true
                 };
             }
-            this._addProfileHeader(this._temporaryTakingSnapshot);
+            this.addProfileHeader(this._temporaryRecordingProfile);
         }
-        ProfilerAgent.takeHeapSnapshot(detailed);
+        ProfilerAgent.takeHeapSnapshot();
+        WebInspector.userMetrics.ProfilesHeapProfileTaken.record();
     },
 
     _reportHeapSnapshotProgress: function(done, total)
     {
-        if (this.hasTemporaryProfile(WebInspector.HeapSnapshotProfileType.TypeId)) {
-            this._temporaryTakingSnapshot.sideBarElement.subtitle = WebInspector.UIString("%.2f%%", (done / total) * 100);
+        if (this.hasTemporaryProfile(WebInspector.DetailedHeapshotProfileType.TypeId)) {
+            this._temporaryRecordingProfile.sidebarElement.subtitle = WebInspector.UIString("%.2f%%", (done / total) * 100);
+            this._temporaryRecordingProfile.sidebarElement.wait = true;
             if (done >= total)
-                this._removeProfileHeader(this._temporaryTakingSnapshot);
+                this._removeTemporaryProfile();
         }
-    },
-
-    handleShortcut: function(event)
-    {
-        if (!Preferences.heapProfilerPresent || Preferences.detailedHeapProfiles)
-            return;
-        var combo = ["U+004C", "U+0045", "U+0041", "U+004B", "U+005A"];  // "LEAKZ"
-        if (this._recognizeKeyboardCombo(combo, event)) {
-            this._displayDetailedHeapProfilesEnabledHint();          
-            this._enableDetailedHeapProfiles(true);
-        }
-    },
-
-    _recognizeKeyboardCombo: function(combo, event)
-    {
-        var isRecognized = false;
-        if (!this._comboPosition) {
-            if (event.keyIdentifier === combo[0])
-                this._comboPosition = 1;
-        } else if (event.keyIdentifier === combo[this._comboPosition]) {
-            if (++this._comboPosition === combo.length)
-                isRecognized = true;
-        } else
-            delete this._comboPosition;
-        if (this._comboPosition)
-            event.handled = true;
-        return isRecognized;
-    },
-    
-    _displayDetailedHeapProfilesEnabledHint: function()
-    {
-        var message = new WebInspector.HelpScreen("Congratulations!");
-        message.contentElement.addStyleClass("help-table");
-        message.contentElement.textContent = "Detailed Heap snapshots are now enabled.";
-        message.show();
-
-        function hideHint()
-        {
-            message._hide();
-        }
-
-        setTimeout(hideHint, 2000);
-    },
-
-    _enableDetailedHeapProfiles: function(resetAgent)
-    {
-        if (resetAgent)
-            this._clearProfiles();
-        else
-            this._reset();
-        var oldProfileType = this._profileTypesByIdMap[WebInspector.HeapSnapshotProfileType.TypeId];
-        var profileType = new WebInspector.DetailedHeapshotProfileType();
-        profileType.treeElement = oldProfileType.treeElement;
-        this._profileTypesByIdMap[profileType.id] = profileType;
-        Preferences.detailedHeapProfiles = true;
-        this.hide();
-        this.show();
     }
 }
 
@@ -780,16 +875,6 @@ WebInspector.ProfilerDispatcher = function(profiler)
 }
 
 WebInspector.ProfilerDispatcher.prototype = {
-    profilerWasEnabled: function()
-    {
-        this._profiler._profilerWasEnabled();
-    },
-
-    profilerWasDisabled: function()
-    {
-        this._profiler._profilerWasDisabled();
-    },
-
     resetProfiles: function()
     {
         this._profiler._reset();
@@ -797,7 +882,7 @@ WebInspector.ProfilerDispatcher.prototype = {
 
     addProfileHeader: function(profile)
     {
-        this._profiler._addProfileHeader(profile);
+        this._profiler.addProfileHeader(profile);
     },
 
     addHeapSnapshotChunk: function(uid, chunk)
@@ -812,7 +897,7 @@ WebInspector.ProfilerDispatcher.prototype = {
 
     setRecordingProfile: function(isProfiling)
     {
-        this._profiler._setRecordingProfile(isProfiling);
+        this._profiler.setRecordingProfile(WebInspector.CPUProfileType.TypeId, isProfiling);
     },
 
     reportHeapSnapshotProgress: function(done, total)
@@ -837,7 +922,8 @@ WebInspector.ProfileSidebarTreeElement = function(profile, titleFormat, classNam
 WebInspector.ProfileSidebarTreeElement.prototype = {
     onselect: function()
     {
-        this.treeOutline.panel.showProfile(this.profile);
+        if (!this._suppressOnSelect)
+            this.treeOutline.panel.showProfile(this.profile);
     },
 
     ondelete: function()
@@ -892,3 +978,29 @@ WebInspector.ProfileGroupSidebarTreeElement.prototype = {
 }
 
 WebInspector.ProfileGroupSidebarTreeElement.prototype.__proto__ = WebInspector.SidebarTreeElement.prototype;
+
+/**
+ * @constructor
+ * @extends {WebInspector.SidebarTreeElement}
+ */
+WebInspector.ProfilesSidebarTreeElement = function(panel)
+{
+    this._panel = panel;
+    this.small = false;
+
+    WebInspector.SidebarTreeElement.call(this, "profile-launcher-view-tree-item", WebInspector.UIString("Profiles"), "", null, false);
+}
+
+WebInspector.ProfilesSidebarTreeElement.prototype = {
+    onselect: function()
+    {
+        this._panel._showLauncherView();
+    },
+
+    get selectable()
+    {
+        return true;
+    }
+}
+
+WebInspector.ProfilesSidebarTreeElement.prototype.__proto__ = WebInspector.SidebarTreeElement.prototype;
